@@ -4,7 +4,7 @@ ptipython: IPython interactive shell with the `prompt_toolkit` front-end.
 Usage:
     ptpython [ --vi ]
              [ --config-dir=<directory> ] [ --interactive=<filename> ]
-             [--] [ <file> <arg>... ]
+             [--] [ <arg>... ]
     ptpython -h | --help
 
 Options:
@@ -26,6 +26,10 @@ def run():
     vi_mode = bool(a['--vi'])
     config_dir = os.path.expanduser(a['--config-dir'] or '~/.ptpython/')
 
+    # Create config directory.
+    if not os.path.isdir(config_dir):
+        os.mkdir(config_dir)
+
     # If IPython is not available, show message and exit here with error status
     # code.
     try:
@@ -38,12 +42,13 @@ def run():
         from ptpython.repl import run_config, enable_deprecation_warnings
 
     # Add the current directory to `sys.path`.
-    sys.path.append('.')
+    if sys.path[0] != '':
+        sys.path.insert(0, '')
 
     # When a file has been given, run that, otherwise start the shell.
-    if a['<file>']:
-        sys.argv = [a['<file>']] + a['<arg>']
-        six.exec_(compile(open(a['<file>'], "rb").read(), a['<file>'], 'exec'))
+    if a['<arg>'] and not a['--interactive']:
+        sys.argv = a['<arg>']
+        six.exec_(compile(open(a['<arg>'][0], "rb").read(), a['<arg>'][0], 'exec'))
     else:
         enable_deprecation_warnings()
 
@@ -55,6 +60,7 @@ def run():
         # --interactive
         if a['--interactive']:
             path = a['--interactive']
+            sys.argv = [a['--interactive']] + a['<arg>']
 
             if os.path.exists(path):
                 with open(path, 'r') as f:
