@@ -13,12 +13,14 @@ from pygments.lexers import PythonTracebackLexer, PythonLexer
 from pygments.styles.default import DefaultStyle
 
 from prompt_toolkit.application import AbortAction
+from prompt_toolkit.enums import DEFAULT_BUFFER
 from prompt_toolkit.interface import AcceptAction, CommandLineInterface
 from prompt_toolkit.layout.utils import token_list_width
 from prompt_toolkit.shortcuts import create_asyncio_eventloop
+from prompt_toolkit.styles import PygmentsStyle
 from prompt_toolkit.utils import DummyContext, Callback
 
-from .python_input import PythonInput
+from .python_input import PythonInput, PythonCommandLineInterface
 from .eventloop import create_eventloop
 
 import os
@@ -79,8 +81,8 @@ class PythonRepl(PythonInput):
 
             # Append to history and reset.
             cli.search_state.text = ''
-            cli.buffers['default'].reset(append_to_history=True)
-            self.key_bindings_manager.reset()
+            cli.buffers[DEFAULT_BUFFER].reset(append_to_history=True)
+            self.key_bindings_manager.reset(cli)
 
     def _execute(self, cli, line):
         """
@@ -96,7 +98,7 @@ class PythonRepl(PythonInput):
 
         if line.lstrip().startswith('\x1a'):
             # When the input starts with Ctrl-Z, quit the REPL.
-            cli.set_return_value(None)
+            cli.exit()
 
         elif line.lstrip().startswith('!'):
             # Run as shell command
@@ -161,7 +163,7 @@ class PythonRepl(PythonInput):
         # (We use the default style. Most other styles result
         # in unreadable colors for the traceback.)
         tokens = _lex_python_traceback(tb)
-        cli.print_tokens(tokens, style=DefaultStyle)
+        cli.print_tokens(tokens, style=PygmentsStyle(DefaultStyle))
 
         output.write('%s\n\n' % e)
         output.flush()
@@ -289,7 +291,7 @@ def embed(globals=None, locals=None, configure=None,
     if configure:
         configure(repl)
 
-    cli = CommandLineInterface(application=repl.create_application(), eventloop=eventloop)
+    cli = PythonCommandLineInterface(python_input=repl, eventloop=eventloop)
 
     # Start repl.
     patch_context = cli.patch_stdout_context() if patch_stdout else DummyContext()
